import matplotlib.pyplot as plt
import numpy as np
import serial
import time
from scipy import interpolate
from math import trunc

ser = serial.Serial('/dev/cu.wchusbserial110', 115200)
ser.flushInput()

ser_bytes = ser.readline()

def handle_close(evt):
    raise SystemExit('Closed figure, exit program.')

def get_data():
    global data, ser
    line = ser.readline().decode().strip()
    line = line.rstrip(line[-1])
    temps = [float(t.strip() or 0) for t in line.split(",")]
    data = np.array(temps).reshape((8, 8))
    return data

pix_res = (8, 8) # pixel resolution
xx,yy = (np.linspace(0, pix_res[0], pix_res[0]), np.linspace(0, pix_res[1], pix_res[1]))
zz = np.zeros(pix_res) # set array with zeros first
# new resolution
pix_mult = 10 # multiplier for interpolation
interp_res = (int(pix_mult*pix_res[0]), int(pix_mult*pix_res[1]))
grid_x, grid_y = (np.linspace(0, pix_res[0], interp_res[0]), np.linspace(0, pix_res[1], interp_res[1]))

# interp function
def interp(z_var):
    # cubic interpolation on the image
    # at a resolution of (pix_mult*8 x pix_mult*8)
    f = interpolate.interp2d(xx, yy, z_var, kind='cubic')
    return f(grid_x, grid_y)

grid_z = interp(zz) # interpolated image

#####################################
# Start and Format Figure
#####################################
#
norm_pix = get_data()
plt.rcParams.update({'font.size':12})
fig_dims = (10, 9) # figure size
fig, ax = plt.subplots(figsize=fig_dims) # start figure
fig.canvas.manager.set_window_title('AMG8833 Image Interpolation')
im1 = ax.imshow(grid_z, vmin=norm_pix.min(), vmax=norm_pix.max(), cmap=plt.cm.RdBu_r) # plot image, with temperature bounds
cbar = fig.colorbar(im1, fraction=0.0475, pad=0.03) # colorbar
cbar.set_label('Temperature [C]', labelpad=25) # temp. label
fig.canvas.draw() # draw figure

ax_bgnd = fig.canvas.copy_from_bbox(ax.bbox) # background for speeding up runs
fig.show() # show figure

def plotUpdate():
    fig.canvas.restore_region(ax_bgnd)  # restore background (speeds up run)
    new_z = interp(np.reshape(get_data(), pix_res))  # interpolated image
    im1.set_data(new_z)  # update plot with new interpolated temps
    im1.set_clim(vmin=new_z.min(), vmax=new_z.max())  # set bounds
    cbar_ticks = np.linspace(new_z.min(), new_z.max(), num=10, endpoint=True).round(1)
    cbar.set_ticks(cbar_ticks)
    cbar.draw_all()
    ax.draw_artist(im1)  # draw image again
    fig.canvas.blit(ax.bbox)  # blitting - for speeding up run
    fig.canvas.flush_events()  # for real-time plot
    plt.pause(0.0001)

try:
    while True:
        plotUpdate()

except KeyboardInterrupt:
        print("CTRL-C: Program Stopping via Keyboard Interrupt...")
        exit()

finally:
        print("Exiting Loop")